@@ -15,29 +15,10 @@ function(sgp_object,
 	simulate.sgps=TRUE,
 	parallel.config=NULL,
 	save.intermediate.results=FALSE,
-	sgp.summaries=list(MEDIAN_SGP="median_na(SGP)",
-		MEDIAN_SGP_TARGET="median_na(SGP_TARGET)",
-		PERCENT_CATCHING_UP_KEEPING_UP="percent_in_category(CATCH_UP_KEEP_UP_STATUS, list(c('Catch Up: Yes', 'Keep Up: Yes')), list(c('Catch Up: Yes', 'Catch Up: No', 'Keep Up: Yes', 'Keep Up: No')))",
-		MEDIAN_SGP_COUNT="num_non_missing(SGP)",
-		PERCENT_AT_ABOVE_PROFICIENT="percent_in_category(ACHIEVEMENT_LEVEL, list(c('Proficient', 'Advanced')), list(c('Unsatisfactory', 'Partially Proficient', 'Proficient', 'Advanced')))",
-		PERCENT_AT_ABOVE_PROFICIENT_COUNT="num_non_missing(ACHIEVEMENT_LEVEL)",
-                PERCENT_AT_ABOVE_PROFICIENT_PRIOR="percent_in_category(ACHIEVEMENT_LEVEL_PRIOR, list(c('Proficient', 'Advanced')), list(c('Unsatisfactory', 'Partially Proficient', 'Proficient', 'Advanced')))",
-                PERCENT_AT_ABOVE_PROFICIENT_PRIOR_COUNT="num_non_missing(ACHIEVEMENT_LEVEL_PRIOR)"),
-	summary.groups=list(institution=c("STATE", "DISTRICT_NUMBER", "SCHOOL_NUMBER"),
-		content="CONTENT_AREA",
-		time="YEAR",
-		institution_level="GRADE",
-		demographic=c("GENDER", "ETHNICITY", "FREE_REDUCED_LUNCH_STATUS", "ELL_STATUS", "IEP_STATUS", "GIFTED_AND_TALENTED_PROGRAM_STATUS", "CATCH_UP_KEEP_UP_STATUS_INITIAL"),
-		institution_inclusion=list(STATE="STATE_ENROLLMENT_STATUS", DISTRICT_NUMBER="DISTRICT_ENROLLMENT_STATUS", SCHOOL_NUMBER="SCHOOL_ENROLLMENT_STATUS")),
-        confidence.interval.groups=list(TYPE="Bootstrap",
-                VARIABLES=c("SGP"),
-                QUANTILES=c(0.025, 0.975),
-                GROUPS=list(institution="SCHOOL_NUMBER",
-                content="CONTENT_AREA",
-                time="YEAR",
-                institution_level= NULL,
-                demographic=NULL,
-                institution_inclusion=list(STATE=NULL, DISTRICT_NUMBER=NULL, SCHOOL_NUMBER="SCHOOL_ENROLLMENT_STATUS"))),
+	sgPlot.demo.report=FALSE,
+	sgp.summaries=NULL,
+	summary.groups=NULL,
+	confidence.interval.groups=NULL,
 	plot.types=c("bubblePlot", "studentGrowthPlot", "growthAchievementPlot")) {
 
         started.at <- proc.time()
@@ -49,13 +30,15 @@ function(sgp_object,
 		tmp.name <- gsub("_", " ", deparse(substitute(sgp_object)))
 		if (any(sapply(c(state.name, "Demonstration", "sgpData LONG"), function(x) regexpr(x, tmp.name)))==1) {
 			state <- c(state.abb, rep("DEMO", 2))[which(sapply(c(state.name, "Demonstration", "sgpData LONG"), function(x) regexpr(x, tmp.name))==1)]
+		} else {
+			message("\tNOTE: Use of the higher level 'abcSGP' function requires extensive metadata embedded in the 'SGPstateData' list object. Please add your state's data to 'SGPstateData' by examining a state that is currently embedded. For example, SGPstateData[['DEMO']]. Please contact the package administrator with further questions.")
 		}
 	}
 
         ### Check for consistency between simulate.sgps and existence of CSEMs ###
 
 	if (simulate.sgps & is.null(SGPstateData[[state]][["Assessment_Program_Information"]][["CSEM"]])) {
-                message("CSEMs are required in SGPstateData to simulate SGPs for confidence interval calculations. Confidence intervals will not be calculated.")
+                message("\tCSEMs are required in SGPstateData to simulate SGPs for confidence interval calculations. Confidence intervals will not be calculated.")
                 simulate.sgps <- FALSE
         }
 
@@ -63,7 +46,7 @@ function(sgp_object,
 	### prepareSGP ###
 
 	if ("prepareSGP" %in% steps) {
-		sgp_object <- prepareSGP(sgp_object, var.names=prepareSGP.var.names)
+		sgp_object <- prepareSGP(sgp_object, state=state, var.names=prepareSGP.var.names)
 	        if (save.intermediate.results) save(sgp_object, file="sgp_object.Rdata")
 	}
 
@@ -82,6 +65,14 @@ function(sgp_object,
 	### analyzeSGP ###
 
 	if ("analyzeSGP" %in% steps) {
+
+        ### Check for consistency between simulate.sgps and existence of CSEMs ###
+
+		if (simulate.sgps & is.null(SGPstateData[[state]][["Assessment_Program_Information"]][["CSEM"]])) {
+        	        message("\tCSEMs are required in SGPstateData to simulate SGPs for confidence interval calculations. Confidence intervals will not be calculated.")
+			simulate.sgps <- FALSE
+		}
+
 		sgp_object <- analyzeSGP(
 			sgp_object=sgp_object,
 			state=state,
@@ -127,7 +118,8 @@ function(sgp_object,
 			content_areas=content_areas, 
 			sgp.summaries=sgp.summaries, 
 			summary.groups=summary.groups, 
-			confidence.interval.groups=confidence.interval.groups)
+			confidence.interval.groups=confidence.interval.groups,
+			parallel.config=parallel.config)
 
                 if (save.intermediate.results) save(sgp_object, file="sgp_object.Rdata")
 	}
@@ -143,10 +135,11 @@ function(sgp_object,
 			state=state,
 			bPlot.years=years,
 			sgPlot.years=years,
-			sgPlot.demo.report=TRUE,
+			sgPlot.demo.report=sgPlot.demo.report,
 			gaPlot.years=years,
 			bPlot.content_areas=content_areas,
-			gaPlot.content_areas=content_areas)
+			gaPlot.content_areas=content_areas,
+			parallel.config=parallel.config)
 	}
 
         message(paste("Finished abcSGP", date(), "in", timetaken(started.at), "\n"))
