@@ -1,29 +1,56 @@
 `outputSGP` <- 
 function(sgp_object,
-	state,
-	output.type="SchoolView_WIDE",
+	state=NULL,
+	output.type="SchoolView",
 	baseline.sgps=FALSE,
-	outputSGP.years=NULL,
-	outputSGP.content_areas=NULL,
-	outputSGP.anonymize=FALSE) {
-
-
-	### Create state (if missing) from sgp_object (if possible)
-
-        if (missing(state)) {
-                tmp.name <- gsub("_", " ", deparse(substitute(sgp_object)))
-                if (any(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name)))==1) {
-                        state <- c(state.abb, "DEMO")[which(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name))==1)]
-                }
-        }
-
-
-if ("SchoolView_WIDE" %in% output.type) {
+	outputSGP_SUMMARY.years=NULL,
+	outputSGP_SUMMARY.content_areas=NULL,
+	outputSGP_INDIVIDUAL.years=NULL,
+	outputSGP_INDIVIDUAL.content_areas=NULL,
+	outputSGP.anonymize=FALSE,
+	outputSGP.student.groups=NULL,
+	outputSGP.directory=file.path("Data", "SchoolView")) {
 
 	### Define varaibles (to prevent R CMD check warnings)
 
 	SCALE_SCORE <- CONTENT_AREA <- YEAR <- GRADE <- ID <- ETHNICITY <- GENDER <- LAST_NAME <- FIRST_NAME <- VALID_CASE <- DISTRICT_NUMBER <- SCHOOL_NUMBER <- NULL
+	names.type <- names.provided <- names.output <- NULL
 
+	### Create state (if missing) from sgp_object (if possible)
+
+	if (is.null(state)) {
+		tmp.name <- gsub("_", " ", deparse(substitute(sgp_object)))
+		if (any(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name)))==1) {
+			state <- c(state.abb, "DEMO")[which(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name))==1)]
+		}
+	}
+
+
+	### Create relevant variables
+
+	if (is.null(outputSGP.student.groups)) {
+		outputSGP.student.groups <- intersect(names(sgp_object@Data), subset(sgp_object@Names, names.type=="demographic" & names.output==TRUE, select=names.provided, drop=TRUE))
+	}
+
+
+if ("SchoolView" %in% output.type) {
+
+	### 
+	### Summary Tables
+	###
+
+	sqliteSGP(
+		sgp_object=sgp_object,
+		state=state,
+		years=outputSGP_SUMMARY.years,
+		content_areas=outputSGP_SUMMARY.content_areas,
+		other.student.groups=outputSGP.student.groups,
+		output.directory=outputSGP.directory)
+	
+
+	###
+	### WIDE Data
+	###
 
 	### Utility functions
 
@@ -36,23 +63,23 @@ if ("SchoolView_WIDE" %in% output.type) {
 		paste(as.numeric(unlist(strsplit(as.character(year), "_")))+increment, collapse="_")
 	}
 
-        get.my.cutscore.year <- function(state, content_area, year) {
-                tmp.cutscore.years <- sapply(strsplit(names(SGPstateData[[state]][["Achievement"]][["Cutscores"]])[grep(content_area, names(SGPstateData[[state]][["Achievement"]][["Cutscores"]]))], "[.]"),
-                        function(x) x[2])
-                if (any(!is.na(tmp.cutscore.years))) {
-                if (year %in% tmp.cutscore.years) {
-                  return(paste(content_area, year, sep="."))
-               } else {
-                  if (year==sort(c(year, tmp.cutscore.years))[1]) {
-                     return(content_area)
-                  } else {
-                     return(paste(content_area, sort(tmp.cutscore.years)[which(year==sort(c(year, tmp.cutscore.years)))-1], sep="."))
-                  }
-               }
-             } else {
-                  return(content_area)
-             }
-        }
+	get.my.cutscore.year <- function(state, content_area, year) {
+		tmp.cutscore.years <- sapply(strsplit(names(SGPstateData[[state]][["Achievement"]][["Cutscores"]])[grep(content_area, names(SGPstateData[[state]][["Achievement"]][["Cutscores"]]))], "[.]"),
+			function(x) x[2])
+		if (any(!is.na(tmp.cutscore.years))) {
+			if (year %in% tmp.cutscore.years) {
+				return(paste(content_area, year, sep="."))
+			} else {
+				if (year==sort(c(year, tmp.cutscore.years))[1]) {
+					return(content_area)
+				} else {
+					return(paste(content_area, sort(tmp.cutscore.years)[which(year==sort(c(year, tmp.cutscore.years)))-1], sep="."))
+				}
+			}
+		} else {
+			return(content_area)
+		}
+	}
 
 	piecewise.transform <- function(scale_score, state, content_area, year, grade, output.digits=1) {
 		if (content_area %in% names(SGPstateData[[state]][["Student_Report_Information"]][["Transformed_Achievement_Level_Cutscores"]]) &
@@ -60,6 +87,7 @@ if ("SchoolView_WIDE" %in% output.type) {
 				ncol=2, byrow=TRUE)[,2])) {
 
 			tmp.loss.hoss <- SGPstateData[[state]][["Achievement"]][["Knots_Boundaries"]][[content_area]][[paste("loss.hoss_", grade, sep="")]]
+			scale_score[scale_score < tmp.loss.hoss[1]] <- tmp.loss.hoss[1]; scale_score[scale_score > tmp.loss.hoss[2]] <- tmp.loss.hoss[2]
 			my.content_area <- get.my.cutscore.year(state, content_area, year)
 			tmp.old.cuts <- c(tmp.loss.hoss[1], SGPstateData[[state]][["Achievement"]][["Cutscores"]][[my.content_area]][[paste("GRADE_", grade, sep="")]], 
 				tmp.loss.hoss[2])
@@ -78,65 +106,65 @@ if ("SchoolView_WIDE" %in% output.type) {
 	#### Set key
 
 	long.key <- c("VALID_CASE", "YEAR", "CONTENT_AREA", "DISTRICT_NUMBER", "SCHOOL_NUMBER")
-	key(sgp_object@Data) <- long.key
+	setkeyv(sgp_object@Data, long.key)
 
 
 	#### Year stuff 
 
-	if (is.null(outputSGP.years)) {
+	if (is.null(outputSGP_INDIVIDUAL.years)) {
 		tmp.years <- sort(unique(sgp_object@Data[J("VALID_CASE")][["YEAR"]]))
 		tmp.last.year <- tail(tmp.years, 1)
 	} else {
 		tmp.all.years <- sort(unique(sgp_object@Data[J("VALID_CASE")][["YEAR"]])) 
-		tmp.years <- tmp.all.years[1:which(tmp.all.years==tail(sort(outputSGP.years), 1))]
+		tmp.years <- tmp.all.years[1:which(tmp.all.years==tail(sort(outputSGP_INDIVIDUAL.years), 1))]
 		tmp.last.year <- tail(tmp.years, 1)
 	}
 
 
 	#### Content area stuff 
 
-	if (is.null(outputSGP.content_areas)) {
+	if (is.null(outputSGP_INDIVIDUAL.content_areas)) {
 		tmp.content_areas <- sort(unique(sgp_object@Data[J("VALID_CASE", tmp.last.year)][["CONTENT_AREA"]]))
 	} else {
-		tmp.content_areas <- as.factor(outputSGP.content_areas)
+		tmp.content_areas <- as.factor(outputSGP_INDIVIDUAL.content_areas)
 	}
 
 
 	### subset data
 
-        tmp.districts.and.schools <- unique(data.table(sgp_object@Data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas)][,
-                                        list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)],
-                                        key=key(sgp_object)))
-        report.ids <- unique(sgp_object@Data[tmp.districts.and.schools][["ID"]])
-        key(sgp_object@Data) <- c("ID", "CONTENT_AREA", "YEAR")
-        tmp.table <- sgp_object@Data[CJ(report.ids, tmp.content_areas, tmp.years)]
+	tmp.districts.and.schools <- unique(data.table(sgp_object@Data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas)][,
+														list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)],
+												key=key(sgp_object)))
+	report.ids <- unique(sgp_object@Data[tmp.districts.and.schools][["ID"]])
+	setkeyv(sgp_object@Data, c("ID", "CONTENT_AREA", "YEAR"))
+	tmp.table <- sgp_object@Data[CJ(report.ids, tmp.content_areas, tmp.years)]
 
 
-	### Create transformed scale scores (NOT necessary if wide data is provided)
+	### Create transformed scale scores
 
-	key(tmp.table) <- c("CONTENT_AREA", "YEAR", "GRADE")
+	setkeyv(tmp.table, c("CONTENT_AREA", "YEAR", "GRADE"))
 	tmp.table$TRANSFORMED_SCALE_SCORE <- tmp.table[,
 		piecewise.transform(SCALE_SCORE, state, as.character(CONTENT_AREA[1]), as.character(YEAR[1]), as.character(GRADE[1])), 
 			by=list(CONTENT_AREA, YEAR, GRADE)]$V1
 
 
 	#### Anonymize (if requested) (NOT necessary if wide data is provided)
-     
-	if (outputSGP.anonymize) {
+ 
+	if (outputSGP.anonymize | !all(c("LAST_NAME", "FIRST_NAME") %in% names(tmp.table))) {
 		require(randomNames)
-                if (!"ETHNICITY" %in% names(tmp.table)) tmp.table[["ETHNICITY"]] <- 1
-                if (!"GENDER" %in% names(tmp.table)) tmp.table[["GENDER"]] <- round(runif(dim(tmp.table)[1], min=0, max=1))
+		if (!"ETHNICITY" %in% names(tmp.table)) tmp.table[["ETHNICITY"]] <- 1
+		if (!"GENDER" %in% names(tmp.table)) tmp.table[["GENDER"]] <- round(runif(dim(tmp.table)[1], min=0, max=1))
 		tmp.dt <- tmp.table[,list(ID, ETHNICITY, GENDER)]
-		key(tmp.dt) <- "ID"
+		setkey(tmp.dt, ID)
 		tmp.dt <- tmp.dt[!duplicated(tmp.dt),]
 
 		tmp.dt$LAST_NAME <- randomNames(gender=tmp.dt$GENDER, ethnicity=tmp.dt$ETHNICITY, which.names="last")
 		tmp.dt$FIRST_NAME <- randomNames(gender=tmp.dt$GENDER, ethnicity=tmp.dt$ETHNICITY, which.names="first")
 
 		names.dt <- tmp.dt[,list(ID, LAST_NAME, FIRST_NAME)]
-		key(names.dt) <- "ID"
+		setkey(names.dt, ID)
 
-		key(tmp.table) <- "ID"
+		setkey(tmp.table, ID)
 		tmp.table <- names.dt[tmp.table]
 	} ## END if (outputSGP.anonymize)
 
@@ -145,7 +173,7 @@ if ("SchoolView_WIDE" %in% output.type) {
 
 	variables.to.keep <- c("VALID_CASE", "ID", "LAST_NAME", "FIRST_NAME", "CONTENT_AREA", "YEAR", "GRADE", "EMH_LEVEL", 
 		"SCALE_SCORE", "TRANSFORMED_SCALE_SCORE", "ACHIEVEMENT_LEVEL", "SGP", "SGP_TARGET", "SCHOOL_NUMBER", "DISTRICT_NUMBER",
-		"GENDER", "ETHNICITY", "FREE_REDUCED_LUNCH_STATUS", "ELL_STATUS", "IEP_STATUS", "GIFTED_AND_TALENTED_PROGRAM_STATUS",
+		outputSGP.student.groups,
 		"SCHOOL_ENROLLMENT_STATUS", "DISTRICT_ENROLLMENT_STATUS", "STATE_ENROLLMENT_STATUS")
 
 	outputSGP.data <- reshape(tmp.table[VALID_CASE=="VALID_CASE", variables.to.keep, with=FALSE],
@@ -160,7 +188,7 @@ if ("SchoolView_WIDE" %in% output.type) {
 	for (j in 1:3) {
 		tmp.proj.names <- paste(tmp.content_areas, tmp.last.year, sep=".")
 		if (all(tmp.proj.names %in% names(sgp_object@SGP[["SGProjections"]]))) {
-			key(outputSGP.data) <- c("ID", "CONTENT_AREA")
+			setkeyv(outputSGP.data, c("ID", "CONTENT_AREA"))
 			tmp.list <- list()
 			for (i in tmp.proj.names) {
 				tmp.list[[i]] <- data.table(CONTENT_AREA=unlist(strsplit(i, "[.]"))[1],
@@ -169,7 +197,7 @@ if ("SchoolView_WIDE" %in% output.type) {
 			outputSGP.data <- data.table(rbind.all(tmp.list), key=paste(key(outputSGP.data), collapse=","))[outputSGP.data]
 			tmp.grade.name <- paste("GRADE", tmp.last.year, sep=".")
 			tmp.year.name <- .year.increment(tmp.last.year, j)
-			key(outputSGP.data) <- c("CONTENT_AREA", tmp.grade.name)
+			setkeyv(outputSGP.data, c("CONTENT_AREA", tmp.grade.name))
 			for (proj.iter in grep(paste("PROJ_YEAR", j, sep="_"), names(outputSGP.data))) {
 			tmp.scale_score.name <- names(outputSGP.data)[proj.iter]
 			outputSGP.data[[proj.iter]] <- outputSGP.data[,
@@ -187,31 +215,28 @@ if ("SchoolView_WIDE" %in% output.type) {
 	
 	## Rename variables to keep
 
-	names(outputSGP.data)[which(names(outputSGP.data)=="ID")] <- "STATE_ASSIGNED_ID"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("LAST_NAME", tmp.last.year, sep="."))] <- "LAST_NAME"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("FIRST_NAME", tmp.last.year, sep="."))] <- "FIRST_NAME"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("DISTRICT_NUMBER", tmp.last.year, sep="."))] <- "DISTRICT_NUMBER"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("SCHOOL_NUMBER", tmp.last.year, sep="."))] <- "SCHOOL_NUMBER"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("EMH_LEVEL", tmp.last.year, sep="."))] <- "EMH_LEVEL"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("ETHNICITY", tmp.last.year, sep="."))] <- "ETHNICITY"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("GENDER", tmp.last.year, sep="."))] <- "GENDER"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("IEP_STATUS", tmp.last.year, sep="."))] <- "IEP_CODE"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("FREE_REDUCED_LUNCH_STATUS", tmp.last.year, sep="."))] <- "FRL_CODE"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("GIFTED_AND_TALENTED_PROGRAM_STATUS", tmp.last.year, sep="."))] <- "GIFTED_CODE"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("ELL_STATUS", tmp.last.year, sep="."))] <- "LANGUAGE_PROFICIENCY"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("SCHOOL_ENROLLMENT_STATUS", tmp.last.year, sep="."))] <- "SCHOOL_ENROLLMENT_STATUS"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("DISTRICT_ENROLLMENT_STATUS", tmp.last.year, sep="."))] <- "DISTRICT_ENROLLMENT_STATUS"
-	names(outputSGP.data)[which(names(outputSGP.data)==paste("STATE_ENROLLMENT_STATUS", tmp.last.year, sep="."))] <- "STATE_ENROLLMENT_STATUS"
+	setnames(outputSGP.data, which(names(outputSGP.data)=="ID"), "STATE_ASSIGNED_ID")
+	setnames(outputSGP.data, which(names(outputSGP.data)==paste("LAST_NAME", tmp.last.year, sep=".")), "LAST_NAME")
+	setnames(outputSGP.data, which(names(outputSGP.data)==paste("FIRST_NAME", tmp.last.year, sep=".")), "FIRST_NAME")
+	setnames(outputSGP.data, which(names(outputSGP.data)==paste("DISTRICT_NUMBER", tmp.last.year, sep=".")), "DISTRICT_NUMBER")
+	setnames(outputSGP.data, which(names(outputSGP.data)==paste("SCHOOL_NUMBER", tmp.last.year, sep=".")), "SCHOOL_NUMBER")
+	setnames(outputSGP.data, which(names(outputSGP.data)==paste("EMH_LEVEL", tmp.last.year, sep=".")), "EMH_LEVEL")
+	for (i in outputSGP.student.groups) {
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste(i, tmp.last.year, sep=".")), i)
+	}
+	setnames(outputSGP.data, which(names(outputSGP.data)==paste("SCHOOL_ENROLLMENT_STATUS", tmp.last.year, sep=".")), "SCHOOL_ENROLLMENT_STATUS")
+	setnames(outputSGP.data, which(names(outputSGP.data)==paste("DISTRICT_ENROLLMENT_STATUS", tmp.last.year, sep=".")), "DISTRICT_ENROLLMENT_STATUS")
+	setnames(outputSGP.data, which(names(outputSGP.data)==paste("STATE_ENROLLMENT_STATUS", tmp.last.year, sep=".")), "STATE_ENROLLMENT_STATUS")
 
 	for (i in seq_along(tmp.years)) {	
-		names(outputSGP.data)[grep(paste("GRADE", rev(tmp.years)[i], sep="."), names(outputSGP.data))] <- paste("GRADE_LEVEL", tmp.order[i], sep="_")
-		names(outputSGP.data)[grep(paste("SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data)) %w/o% 
-			grep(paste("TRANSFORMED_SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data))] <- 
-			paste("SCALE_SCORE", tmp.order[i], sep="_")
-		names(outputSGP.data)[grep(paste("TRANSFORMED_SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data))] <- paste("TRANSFORMED_SCALE_SCORE", tmp.order[i], sep="_")
-		names(outputSGP.data)[grep(paste("SGP_TARGET", rev(tmp.years)[i], sep="."), names(outputSGP.data))] <- paste("GROWTH_TARGET", tmp.order[i], sep="_")
-		names(outputSGP.data)[grep(paste("SGP", rev(tmp.years)[i], sep="."), names(outputSGP.data))] <- paste("GROWTH_PERCENTILE", tmp.order[i], sep="_")
-		names(outputSGP.data)[grep(paste("ACHIEVEMENT_LEVEL", rev(tmp.years)[i], sep="."), names(outputSGP.data))] <- paste("PERFORMANCE_LEVEL", tmp.order[i], sep="_")
+		setnames(outputSGP.data, grep(paste("GRADE", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("GRADE_LEVEL", tmp.order[i], sep="_"))
+		setnames(outputSGP.data, grep(paste("SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data)) %w/o% 
+			grep(paste("TRANSFORMED_SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data)), 
+			paste("SCALE_SCORE", tmp.order[i], sep="_"))
+		setnames(outputSGP.data, grep(paste("TRANSFORMED_SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("TRANSFORMED_SCALE_SCORE", tmp.order[i], sep="_"))
+		setnames(outputSGP.data, grep(paste("SGP_TARGET", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("GROWTH_TARGET", tmp.order[i], sep="_"))
+		setnames(outputSGP.data, grep(paste("SGP", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("GROWTH_PERCENTILE", tmp.order[i], sep="_"))
+		setnames(outputSGP.data, grep(paste("ACHIEVEMENT_LEVEL", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("PERFORMANCE_LEVEL", tmp.order[i], sep="_"))
 	}
 	
 	## NULLify variable to be removed
@@ -222,12 +247,9 @@ if ("SchoolView_WIDE" %in% output.type) {
 		outputSGP.data[[paste("DISTRICT_NUMBER", i, sep=".")]] <- NULL
 		outputSGP.data[[paste("SCHOOL_NUMBER", i, sep=".")]] <- NULL
 		outputSGP.data[[paste("EMH_LEVEL", i, sep=".")]] <- NULL
-		outputSGP.data[[paste("ETHNICITY", i, sep=".")]] <- NULL
-		outputSGP.data[[paste("GENDER", i, sep=".")]] <- NULL
-		outputSGP.data[[paste("IEP_STATUS", i, sep=".")]] <- NULL
-		outputSGP.data[[paste("FREE_REDUCED_LUNCH_STATUS", i, sep=".")]] <- NULL
-		outputSGP.data[[paste("GIFTED_AND_TALENTED_PROGRAM_STATUS", i, sep=".")]] <- NULL
-		outputSGP.data[[paste("ELL_STATUS", i, sep=".")]] <- NULL
+		for (j in outputSGP.student.groups) {
+			outputSGP.data[[paste(j, i, sep=".")]] <- NULL
+		}
 		outputSGP.data[[paste("SCHOOL_ENROLLMENT_STATUS", i, sep=".")]] <- NULL
 		outputSGP.data[[paste("DISTRICT_ENROLLMENT_STATUS", i, sep=".")]] <- NULL
 		outputSGP.data[[paste("STATE_ENROLLMENT_STATUS", i, sep=".")]] <- NULL
@@ -255,7 +277,7 @@ if ("SchoolView_WIDE" %in% output.type) {
 
 	if (length(names(outputSGP.data)[grep("YEAR_1", names(outputSGP.data))]) == 4) {
 		for (i in 1:3) {
-			names(outputSGP.data)[grep(paste("YEAR", i, sep="_"), names(outputSGP.data))] <- paste("CUT", c(1, 35, 65, 99), "YEAR", i, sep="_")
+			setnames(outputSGP.data, grep(paste("YEAR", i, sep="_"), names(outputSGP.data)), paste("CUT", c(1, 35, 65, 99), "YEAR", i, sep="_"))
 			for (j in c(20, 40, 60, 80)) {
 				outputSGP.data[[paste("CUT", j, "YEAR", i, sep="_")]] <- NA	
 			}
@@ -263,9 +285,9 @@ if ("SchoolView_WIDE" %in% output.type) {
 	}
 	if (length(names(outputSGP.data)[grep("YEAR_1", names(outputSGP.data))]) == 6) {
 		for (i in 1:3) {
-			names(outputSGP.data)[grep(paste("YEAR", i, sep="_"), names(outputSGP.data))] <- paste("CUT", c(1, 20, 40, 60, 80, 99), "YEAR", i, sep="_")
+			setnames(outputSGP.data, grep(paste("YEAR", i, sep="_"), names(outputSGP.data)), paste("CUT", c(1, 20, 40, 60, 80, 99), "YEAR", i, sep="_"))
 			for (j in c(35, 65)) {
-				outputSGP.data[[paste("CUT", j, "YEAR", i, sep="_")]] <- NA	
+				outputSGP.data[[paste("CUT", j, "YEAR", i, sep="_")]] <- NA
 			}
 		}
 	}
@@ -273,8 +295,8 @@ if ("SchoolView_WIDE" %in% output.type) {
 	## Rearrange variables
 
 	tmp.variable.names <- c("STUDENT_GROWTH_ID", "STATE_ASSIGNED_ID", "LAST_NAME", "FIRST_NAME", "MIDDLE_NAME", 
-		"CONTENT_AREA", "YEAR", "DISTRICT_NUMBER", "SCHOOL_NUMBER", "EMH_LEVEL", 
-		"ETHNICITY", "GENDER", "IEP_CODE", "FRL_CODE", "HLS_CODE", "GIFTED_CODE", "LANGUAGE_PROFICIENCY",
+		"CONTENT_AREA", "YEAR", "DISTRICT_NUMBER", "SCHOOL_NUMBER", "EMH_LEVEL",
+		outputSGP.student.groups,
 		"OCTOBER_ENROLLMENT_STATUS", "SCHOOL_ENROLLMENT_STATUS", "DISTRICT_ENROLLMENT_STATUS", "STATE_ENROLLMENT_STATUS",
 		"GRADE_LEVEL_CY", "SCALE_SCORE_CY", "TRANSFORMED_SCALE_SCORE_CY", "PERFORMANCE_LEVEL_CY", "GROWTH_PERCENTILE_CY", "GROWTH_TARGET_CY",
 		"GRADE_LEVEL_PY1", "SCALE_SCORE_PY1", "TRANSFORMED_SCALE_SCORE_PY1", "PERFORMANCE_LEVEL_PY1", "GROWTH_PERCENTILE_PY1", "GROWTH_TARGET_PY1",
@@ -288,9 +310,10 @@ if ("SchoolView_WIDE" %in% output.type) {
 		"CUT_1_YEAR_2", "CUT_99_YEAR_2", "CUT_35_YEAR_2", "CUT_65_YEAR_2", "CUT_20_YEAR_2", "CUT_40_YEAR_2", "CUT_60_YEAR_2", "CUT_80_YEAR_2",
 		"CUT_1_YEAR_3", "CUT_99_YEAR_3", "CUT_35_YEAR_3", "CUT_65_YEAR_3", "CUT_20_YEAR_3", "CUT_40_YEAR_3", "CUT_60_YEAR_3", "CUT_80_YEAR_3")
 
+	write.table(outputSGP.data[,tmp.variable.names, with=FALSE], file=file.path(outputSGP.directory, "SchoolView_WIDE.dat"), row.names=FALSE, na="", quote=FALSE, sep="|")
 	outputSGP.data[,tmp.variable.names, with=FALSE]
 
-} ## End if SchoolView_WIDE %in% output.type
+} ## End if SchoolView %in% output.type
 
 
 
