@@ -1,5 +1,5 @@
-setClassUnion("date", c("POSIXct", "POSIXt"))
-setClass("splineMatrix", contains='matrix', representation(Knots="list", Boundaries="list", Date="date"))
+setClassUnion("list.null", c("list","NULL"))
+setClass("splineMatrix", contains='matrix', representation(Knots="list", Boundaries="list", Version="list.null"))
 .Valid.splineMatrix <- function(object) {
        out <- NULL
        if (is.null(out)) out <- TRUE
